from typing import Optional, List, Dict

from django.http.response import JsonResponse as DJJsonResponse

from gando.config import SETTINGS

from .schemas import ResponseSchema


class JsonResponse(DJJsonResponse):
    def __init__(
        self,

        data: Optional[dict | list | str] = None,

        log_messages__: Optional[List[Dict[str, str]]] = None,
        info_messages__: Optional[List[Dict[str, str]]] = None,
        warning_messages__: Optional[List[Dict[str, str]]] = None,
        error_messages__: Optional[List[Dict[str, str]]] = None,
        exception_messages__: Optional[List[Dict[str, str]]] = None,

        monitor__: dict = None,

        **kwargs,
    ):
        data, many, monitor, msgs = self.__data_parser(
            data,
            monitor__=monitor__,
            log_messages__=log_messages__,
            info_messages__=info_messages__,
            warning_messages__=warning_messages__,
            error_messages__=error_messages__,
            exception_messages__=exception_messages__,
        )

        context = {
            'success': not (bool(msgs['error_messages']) and bool(msgs['exception_messages'])),
            'has_warning': bool(msgs['warning_messages']),
            'monitor': monitor,
            'messages': {
                'log': msgs['log_messages'] or [] if SETTINGS.DEBUG else [],
                'info': msgs['info_messages'] or [],
                'warning': msgs['warning_messages'] or [],
                'error': msgs['error_messages'] or [],
                'exception': msgs['exception_messages'] or [],
            },
            'data': data,
            'many': many,
        }
        context = context or {}
        data = ResponseSchema(**context).model_dump()
        super(JsonResponse, self).__init__(
            data,
            status=kwargs.get('status', 200)
        )

    def __data_parser(self, data: Optional[dict | list | str], **kwargs):

        monitor = kwargs.get('monitor__', {})
        many = False
        msgs = {
            'log_messages': kwargs.get('log_messages__', []),
            'info_messages': kwargs.get('info_messages__', []),
            'warning_messages': kwargs.get('warning_messages__', []),
            'error_messages': kwargs.get('error_messages__', []),
            'exception_messages': kwargs.get('exception_messages__', []),
        }
        msgs__ = {}

        if isinstance(data, str):
            data_response = {'result': data}

        elif isinstance(data, list):
            data_response = {
                'count': len(data),
                'next': None,
                'previous': None,
                'results': data,
            }
            many = True

        elif isinstance(data, dict):
            data, monitor = self.___monitor_detector(data)
            data, msgs__ = self.__messages_detector(data)
            if bool(data.get('results')):
                many = True
                data_response = data

            else:
                data_response = {'result': data}

        else:
            data_response = {'result': {}}

        msgs['log_messages'] += msgs__.get('log_messages', [])
        msgs['info_messages'] += msgs__.get('info_messages', [])
        msgs['warning_messages'] += msgs__.get('warning_messages', [])
        msgs['error_messages'] += msgs__.get('error_messages', [])
        msgs['exception_messages'] += msgs__.get('exception_messages', [])

        ret = data_response, many, monitor, msgs
        return ret

    def ___monitor_detector(self, data):
        monitor = {}
        for i in SETTINGS.MONITOR_KEYS:
            mntr = data.pop(i, None)
            if mntr is not None:
                monitor[i] = mntr
        return data, monitor

    def __messages_detector(self, data: dict) -> (dict, dict):
        msg = {
            'log_messages': data.pop('log_messages', []),
            'info_messages': data.pop('info_messages', []),
            'warning_messages': data.pop('warning_messages', []),
            'error_messages': data.pop('error_messages', []),
            'exception_messages': data.pop('exception_messages', []),
        }
        return data, msg



from django.utils.deprecation import MiddlewareMixin

from gando.http.response import JsonResponse as Response

from .__request_monitor import Monitor
from .__request_response_message import ResponseMessages


class JsonResponse(MiddlewareMixin):

    def process_request(self, request):
        request.monitor = Monitor()
        request.response_messages = ResponseMessages()

    def process_response(self, request, response):
        msg = request.response_messages.export()
        dmsg = msg.model_dump()
        rsp = Response(status=self.__get_status_code(response), **dmsg, **response.__dict__)
        return rsp

    def __get_status_code(self, response):
        try:
            status_code = getattr(response, 'status')
        except:
            status_code = getattr(response, 'status_code')
        return status_code



from typing import List, Dict, Optional

from pydantic import BaseModel


class Messages(BaseModel):
    log: Optional[List[Dict[str, str]]] = list()
    info: Optional[List[Dict[str, str]]] = list()
    warning: Optional[List[Dict[str, str]]] = list()
    error: Optional[List[Dict[str, str]]] = list()
    exception: Optional[List[Dict[str, str]]] = list()



from typing import List, Dict, Optional

from pydantic import BaseModel


class MessagesSchema(BaseModel):
    log_messages: Optional[List[Dict[str, str]]] = list()
    info_messages: Optional[List[Dict[str, str]]] = list()
    warning_messages: Optional[List[Dict[str, str]]] = list()
    error_messages: Optional[List[Dict[str, str]]] = list()
    exception_messages: Optional[List[Dict[str, str]]] = list()


class ResponseMessages:
    log: list = []
    info: list = []
    warning: list = []
    error: list = []
    exception: list = []

    def __int__(self):
        self.log = []
        self.info = []
        self.warning = []
        self.error = []
        self.exception = []

    def add2logs(self, msg_title: str, msg_content: str):
        self.log.append({msg_title: msg_content})

    def add2infos(self, msg_title: str, msg_content: str):
        self.info.append({msg_title: msg_content})

    def add2warnings(self, msg_title: str, msg_content: str):
        self.warning.append({msg_title: msg_content})

    def add2errors(self, msg_title: str, msg_content: str):
        self.error.append({msg_title: msg_content})

    def add2exceptions(self, msg_title: str, msg_content: str):
        self.exception.append({msg_title: msg_content})

    def export(self):
        ret = MessagesSchema(
            log_messages=self.log,
            info_messages=self.info,
            warning_messages=self.warning,
            error_messages=self.error,
            exception_messages=self.exception,
        )
        return ret



from typing import Any


class Monitor:
    monitor: dict = {}

    def __int__(self):
        self.monitor = {}

    def add(self, key: str, value: Any):
        self.monitor[key] = value

    def delete(self, key):
        self.monitor.pop(key, None)

    def export(self):
        return self.monitor



import setuptools

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='gando',
    author='Hydra',
    author_email='navidsoleymani@ymail.com',
    description='A framework based on Django that has tried to gather together the tools needed in the process of creating a large project.',
    keywords='django',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/navidsoleymani/gando.git',
    project_urls={
        'Documentation': 'https://github.com/navidsoleymani/gando.git',
        'Bug Reports':
            'https://github.com/navidsoleymani/gando.git/issues',
        'Source Code': 'https://github.com/navidsoleymani/gando.git',
    },
    package_dir={'': 'src'},
    packages=setuptools.find_packages(where='src'),
    classifiers=[
        'Framework :: Django :: 4.2',
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.8',
    install_requires=[
        'Django',
        'pydantic',
        'djangorestframework',
        'markdown',
        'django-filter',
    ],
)
